@@ -1,9 +1,10 @@
 # coding: utf-8
 """Plotting library."""
+
 import math
 from copy import deepcopy
 from io import BytesIO
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Tuple, Union
 
 import numpy as np
 
@@ -12,13 +13,16 @@ from .compat import GRAPHVIZ_INSTALLED, MATPLOTLIB_INSTALLED, pd_DataFrame
 from .sklearn import LGBMModel
 
 __all__ = [
-    'create_tree_digraph',
-    'plot_importance',
-    'plot_metric',
-    'plot_split_value_histogram',
-    'plot_tree',
+    "create_tree_digraph",
+    "plot_importance",
+    "plot_metric",
+    "plot_split_value_histogram",
+    "plot_tree",
 ]
 
+if TYPE_CHECKING:
+    import matplotlib
+
 
 def _check_not_tuple_of_2_elements(obj: Any, obj_name: str) -> None:
     """Check object is not tuple or does not have 2 elements."""
@@ -27,28 +31,26 @@ def _check_not_tuple_of_2_elements(obj: Any, obj_name: str) -> None:
 
 
 def _float2str(value: float, precision: Optional[int]) -> str:
-    return (f"{value:.{precision}f}"
-            if precision is not None and not isinstance(value, str)
-            else str(value))
+    return f"{value:.{precision}f}" if precision is not None and not isinstance(value, str) else str(value)
 
 
 def plot_importance(
     booster: Union[Booster, LGBMModel],
-    ax=None,
+    ax: "Optional[matplotlib.axes.Axes]" = None,
     height: float = 0.2,
     xlim: Optional[Tuple[float, float]] = None,
     ylim: Optional[Tuple[float, float]] = None,
-    title: Optional[str] = 'Feature importance',
-    xlabel: Optional[str] = 'Feature importance',
-    ylabel: Optional[str] = 'Features',
-    importance_type: str = 'auto',
+    title: Optional[str] = "Feature importance",
+    xlabel: Optional[str] = "Feature importance",
+    ylabel: Optional[str] = "Features",
+    importance_type: str = "auto",
     max_num_features: Optional[int] = None,
     ignore_zero: bool = True,
     figsize: Optional[Tuple[float, float]] = None,
     dpi: Optional[int] = None,
     grid: bool = True,
     precision: Optional[int] = 3,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Any:
     """Plot model's feature importances.
 
@@ -104,7 +106,7 @@ def plot_importance(
     if MATPLOTLIB_INSTALLED:
         import matplotlib.pyplot as plt
     else:
-        raise ImportError('You must install matplotlib and restart your session to plot importance.')
+        raise ImportError("You must install matplotlib and restart your session to plot importance.")
 
     if isinstance(booster, LGBMModel):
         if importance_type == "auto":
@@ -114,7 +116,7 @@ def plot_importance(
         if importance_type == "auto":
             importance_type = "split"
     else:
-        raise TypeError('booster must be Booster or LGBMModel.')
+        raise TypeError("booster must be Booster or LGBMModel.")
 
     importance = booster.feature_importance(importance_type=importance_type)
     feature_name = booster.feature_name()
@@ -131,28 +133,26 @@ def plot_importance(
 
     if ax is None:
         if figsize is not None:
-            _check_not_tuple_of_2_elements(figsize, 'figsize')
+            _check_not_tuple_of_2_elements(figsize, "figsize")
         _, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
 
     ylocs = np.arange(len(values))
-    ax.barh(ylocs, values, align='center', height=height, **kwargs)
+    ax.barh(ylocs, values, align="center", height=height, **kwargs)
 
     for x, y in zip(values, ylocs):
-        ax.text(x + 1, y,
-                _float2str(x, precision) if importance_type == 'gain' else x,
-                va='center')
+        ax.text(x + 1, y, _float2str(x, precision) if importance_type == "gain" else x, va="center")
 
     ax.set_yticks(ylocs)
     ax.set_yticklabels(labels)
 
     if xlim is not None:
-        _check_not_tuple_of_2_elements(xlim, 'xlim')
+        _check_not_tuple_of_2_elements(xlim, "xlim")
     else:
         xlim = (0, max(values) * 1.1)
     ax.set_xlim(xlim)
 
     if ylim is not None:
-        _check_not_tuple_of_2_elements(ylim, 'ylim')
+        _check_not_tuple_of_2_elements(ylim, "ylim")
     else:
         ylim = (-1, len(values))
     ax.set_ylim(ylim)
@@ -160,7 +160,7 @@ def plot_importance(
     if title is not None:
         ax.set_title(title)
     if xlabel is not None:
-        xlabel = xlabel.replace('@importance_type@', importance_type)
+        xlabel = xlabel.replace("@importance_type@", importance_type)
         ax.set_xlabel(xlabel)
     if ylabel is not None:
         ax.set_ylabel(ylabel)
@@ -172,17 +172,17 @@ def plot_split_value_histogram(
     booster: Union[Booster, LGBMModel],
     feature: Union[int, str],
     bins: Union[int, str, None] = None,
-    ax=None,
+    ax: "Optional[matplotlib.axes.Axes]" = None,
     width_coef: float = 0.8,
     xlim: Optional[Tuple[float, float]] = None,
     ylim: Optional[Tuple[float, float]] = None,
-    title: Optional[str] = 'Split value histogram for feature with @index/name@ @feature@',
-    xlabel: Optional[str] = 'Feature split value',
-    ylabel: Optional[str] = 'Count',
+    title: Optional[str] = "Split value histogram for feature with @index/name@ @feature@",
+    xlabel: Optional[str] = "Feature split value",
+    ylabel: Optional[str] = "Count",
     figsize: Optional[Tuple[float, float]] = None,
     dpi: Optional[int] = None,
     grid: bool = True,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Any:
     """Plot split value histogram for the specified feature of the model.
 
@@ -238,29 +238,28 @@ def plot_split_value_histogram(
         import matplotlib.pyplot as plt
         from matplotlib.ticker import MaxNLocator
     else:
-        raise ImportError('You must install matplotlib and restart your session to plot split value histogram.')
+        raise ImportError("You must install matplotlib and restart your session to plot split value histogram.")
 
     if isinstance(booster, LGBMModel):
         booster = booster.booster_
     elif not isinstance(booster, Booster):
-        raise TypeError('booster must be Booster or LGBMModel.')
+        raise TypeError("booster must be Booster or LGBMModel.")
 
     hist, split_bins = booster.get_split_value_histogram(feature=feature, bins=bins, xgboost_style=False)
     if np.count_nonzero(hist) == 0:
-        raise ValueError('Cannot plot split value histogram, '
-                         f'because feature {feature} was not used in splitting')
+        raise ValueError("Cannot plot split value histogram, " f"because feature {feature} was not used in splitting")
     width = width_coef * (split_bins[1] - split_bins[0])
     centred = (split_bins[:-1] + split_bins[1:]) / 2
 
     if ax is None:
         if figsize is not None:
-            _check_not_tuple_of_2_elements(figsize, 'figsize')
+            _check_not_tuple_of_2_elements(figsize, "figsize")
         _, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
 
-    ax.bar(centred, hist, align='center', width=width, **kwargs)
+    ax.bar(centred, hist, align="center", width=width, **kwargs)
 
     if xlim is not None:
-        _check_not_tuple_of_2_elements(xlim, 'xlim')
+        _check_not_tuple_of_2_elements(xlim, "xlim")
     else:
         range_result = split_bins[-1] - split_bins[0]
         xlim = (split_bins[0] - range_result * 0.2, split_bins[-1] + range_result * 0.2)
@@ -268,14 +267,14 @@ def plot_split_value_histogram(
 
     ax.yaxis.set_major_locator(MaxNLocator(integer=True))
     if ylim is not None:
-        _check_not_tuple_of_2_elements(ylim, 'ylim')
+        _check_not_tuple_of_2_elements(ylim, "ylim")
     else:
         ylim = (0, max(hist) * 1.1)
     ax.set_ylim(ylim)
 
     if title is not None:
-        title = title.replace('@feature@', str(feature))
-        title = title.replace('@index/name@', ('name' if isinstance(feature, str) else 'index'))
+        title = title.replace("@feature@", str(feature))
+        title = title.replace("@index/name@", ("name" if isinstance(feature, str) else "index"))
         ax.set_title(title)
     if xlabel is not None:
         ax.set_xlabel(xlabel)
@@ -289,15 +288,15 @@ def plot_metric(
     booster: Union[Dict, LGBMModel],
     metric: Optional[str] = None,
     dataset_names: Optional[List[str]] = None,
-    ax=None,
+    ax: "Optional[matplotlib.axes.Axes]" = None,
     xlim: Optional[Tuple[float, float]] = None,
     ylim: Optional[Tuple[float, float]] = None,
-    title: Optional[str] = 'Metric during training',
-    xlabel: Optional[str] = 'Iterations',
-    ylabel: Optional[str] = '@metric@',
+    title: Optional[str] = "Metric during training",
+    xlabel: Optional[str] = "Iterations",
+    ylabel: Optional[str] = "@metric@",
     figsize: Optional[Tuple[float, float]] = None,
     dpi: Optional[int] = None,
-    grid: bool = True
+    grid: bool = True,
 ) -> Any:
     """Plot one metric during training.
 
@@ -345,31 +344,33 @@ def plot_metric(
     if MATPLOTLIB_INSTALLED:
         import matplotlib.pyplot as plt
     else:
-        raise ImportError('You must install matplotlib and restart your session to plot metric.')
+        raise ImportError("You must install matplotlib and restart your session to plot metric.")
 
     if isinstance(booster, LGBMModel):
         eval_results = deepcopy(booster.evals_result_)
     elif isinstance(booster, dict):
         eval_results = deepcopy(booster)
     elif isinstance(booster, Booster):
-        raise TypeError("booster must be dict or LGBMModel. To use plot_metric with Booster type, first record the metrics using record_evaluation callback then pass that to plot_metric as argument `booster`")
+        raise TypeError(
+            "booster must be dict or LGBMModel. To use plot_metric with Booster type, first record the metrics using record_evaluation callback then pass that to plot_metric as argument `booster`"
+        )
     else:
-        raise TypeError('booster must be dict or LGBMModel.')
+        raise TypeError("booster must be dict or LGBMModel.")
 
     num_data = len(eval_results)
 
     if not num_data:
-        raise ValueError('eval results cannot be empty.')
+        raise ValueError("eval results cannot be empty.")
 
     if ax is None:
         if figsize is not None:
-            _check_not_tuple_of_2_elements(figsize, 'figsize')
+            _check_not_tuple_of_2_elements(figsize, "figsize")
         _, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
 
     if dataset_names is None:
         dataset_names_iter = iter(eval_results.keys())
     elif not isinstance(dataset_names, (list, tuple, set)) or not dataset_names:
-        raise ValueError('dataset_names should be iterable and cannot be empty')
+        raise ValueError("dataset_names should be iterable and cannot be empty")
     else:
         dataset_names_iter = iter(dataset_names)
 
@@ -382,7 +383,7 @@ def plot_metric(
         metric, results = metrics_for_one.popitem()
     else:
         if metric not in metrics_for_one:
-            raise KeyError('No given metric in eval results.')
+            raise KeyError("No given metric in eval results.")
         results = metrics_for_one[metric]
     num_iteration = len(results)
     max_result = max(results)
@@ -393,20 +394,20 @@ def plot_metric(
     for name in dataset_names_iter:
         metrics_for_one = eval_results[name]
         results = metrics_for_one[metric]
-        max_result = max(max(results), max_result)
-        min_result = min(min(results), min_result)
+        max_result = max(*results, max_result)
+        min_result = min(*results, min_result)
         ax.plot(x_, results, label=name)
 
-    ax.legend(loc='best')
+    ax.legend(loc="best")
 
     if xlim is not None:
-        _check_not_tuple_of_2_elements(xlim, 'xlim')
+        _check_not_tuple_of_2_elements(xlim, "xlim")
     else:
         xlim = (0, num_iteration)
     ax.set_xlim(xlim)
 
     if ylim is not None:
-        _check_not_tuple_of_2_elements(ylim, 'ylim')
+        _check_not_tuple_of_2_elements(ylim, "ylim")
     else:
         range_result = max_result - min_result
         ylim = (min_result - range_result * 0.2, max_result + range_result * 0.2)
@@ -417,7 +418,7 @@ def plot_metric(
     if xlabel is not None:
         ax.set_xlabel(xlabel)
     if ylabel is not None:
-        ylabel = ylabel.replace('@metric@', metric)
+        ylabel = ylabel.replace("@metric@", metric)
         ax.set_ylabel(ylabel)
     ax.grid(grid)
     return ax
@@ -432,19 +433,20 @@ def _determine_direction_for_numeric_split(
     missing_type = _MissingType(missing_type_str)
     if math.isnan(fval) and missing_type != _MissingType.NAN:
         fval = 0.0
-    if ((missing_type == _MissingType.ZERO and _is_zero(fval))
-            or (missing_type == _MissingType.NAN and math.isnan(fval))):
-        direction = 'left' if default_left else 'right'
+    if (missing_type == _MissingType.ZERO and _is_zero(fval)) or (
+        missing_type == _MissingType.NAN and math.isnan(fval)
+    ):
+        direction = "left" if default_left else "right"
     else:
-        direction = 'left' if fval <= threshold else 'right'
+        direction = "left" if fval <= threshold else "right"
     return direction
 
 
 def _determine_direction_for_categorical_split(fval: float, thresholds: str) -> str:
     if math.isnan(fval) or int(fval) < 0:
-        return 'right'
-    int_thresholds = {int(t) for t in thresholds.split('||')}
-    return 'left' if int(fval) in int_thresholds else 'right'
+        return "right"
+    int_thresholds = {int(t) for t in thresholds.split("||")}
+    return "left" if int(fval) in int_thresholds else "right"
 
 
 def _to_graphviz(
@@ -456,7 +458,7 @@ def _to_graphviz(
     constraints: Optional[List[int]],
     example_case: Optional[Union[np.ndarray, pd_DataFrame]],
     max_category_values: int,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Any:
     """Convert specified tree to graphviz instance.
 
@@ -466,120 +468,124 @@ def _to_graphviz(
     if GRAPHVIZ_INSTALLED:
         from graphviz import Digraph
     else:
-        raise ImportError('You must install graphviz and restart your session to plot tree.')
+        raise ImportError("You must install graphviz and restart your session to plot tree.")
 
     def add(
-        root: Dict[str, Any],
-        total_count: int,
-        parent: Optional[str],
-        decision: Optional[str],
-        highlight: bool
+        root: Dict[str, Any], total_count: int, parent: Optional[str], decision: Optional[str], highlight: bool
     ) -> None:
         """Recursively add node or edge."""
-        fillcolor = 'white'
-        style = ''
+        fillcolor = "white"
+        style = ""
         tooltip = None
         if highlight:
-            color = 'blue'
-            penwidth = '3'
+            color = "blue"
+            penwidth = "3"
         else:
-            color = 'black'
-            penwidth = '1'
-        if 'split_index' in root:  # non-leaf
+            color = "black"
+            penwidth = "1"
+        if "split_index" in root:  # non-leaf
             shape = "rectangle"
-            l_dec = 'yes'
-            r_dec = 'no'
-            threshold = root['threshold']
-            if root['decision_type'] == '<=':
+            l_dec = "yes"
+            r_dec = "no"
+            threshold = root["threshold"]
+            if root["decision_type"] == "<=":
                 operator = "&#8804;"
-            elif root['decision_type'] == '==':
+            elif root["decision_type"] == "==":
                 operator = "="
             else:
-                raise ValueError('Invalid decision type in tree model.')
+                raise ValueError("Invalid decision type in tree model.")
             name = f"split{root['split_index']}"
-            split_feature = root['split_feature']
+            split_feature = root["split_feature"]
             if feature_names is not None:
                 label = f"<B>{feature_names[split_feature]}</B> {operator}"
             else:
                 label = f"feature <B>{split_feature}</B> {operator} "
             direction = None
             if example_case is not None:
-                if root['decision_type'] == '==':
+                if root["decision_type"] == "==":
                     direction = _determine_direction_for_categorical_split(
-                        fval=example_case[split_feature],
-                        thresholds=root['threshold']
+                        fval=example_case[split_feature], thresholds=root["threshold"]
                     )
                 else:
                     direction = _determine_direction_for_numeric_split(
                         fval=example_case[split_feature],
-                        threshold=root['threshold'],
-                        missing_type_str=root['missing_type'],
-                        default_left=root['default_left']
+                        threshold=root["threshold"],
+                        missing_type_str=root["missing_type"],
+                        default_left=root["default_left"],
                     )
-            if root['decision_type'] == '==':
-                category_values = root['threshold'].split('||')
+            if root["decision_type"] == "==":
+                category_values = root["threshold"].split("||")
                 if len(category_values) > max_category_values:
-                    tooltip = root['threshold']
-                    threshold = '||'.join(category_values[:2]) + '||...||' + category_values[-1]
+                    tooltip = root["threshold"]
+                    threshold = "||".join(category_values[:2]) + "||...||" + category_values[-1]
 
             label += f"<B>{_float2str(threshold, precision)}</B>"
-            for info in ['split_gain', 'internal_value', 'internal_weight', "internal_count", "data_percentage"]:
+            for info in ["split_gain", "internal_value", "internal_weight", "internal_count", "data_percentage"]:
                 if info in show_info:
-                    output = info.split('_')[-1]
-                    if info in {'split_gain', 'internal_value', 'internal_weight'}:
+                    output = info.split("_")[-1]
+                    if info in {"split_gain", "internal_value", "internal_weight"}:
                         label += f"<br/>{_float2str(root[info], precision)} {output}"
-                    elif info == 'internal_count':
+                    elif info == "internal_count":
                         label += f"<br/>{output}: {root[info]}"
                     elif info == "data_percentage":
                         label += f"<br/>{_float2str(root['internal_count'] / total_count * 100, 2)}% of data"
 
             if constraints:
-                if constraints[root['split_feature']] == 1:
+                if constraints[root["split_feature"]] == 1:
                     fillcolor = "#ddffdd"  # light green
-                if constraints[root['split_feature']] == -1:
+                if constraints[root["split_feature"]] == -1:
                     fillcolor = "#ffdddd"  # light red
                 style = "filled"
             label = f"<{label}>"
             add(
-                root=root['left_child'],
+                root=root["left_child"],
                 total_count=total_count,
                 parent=name,
                 decision=l_dec,
-                highlight=highlight and direction == "left"
+                highlight=highlight and direction == "left",
             )
             add(
-                root=root['right_child'],
+                root=root["right_child"],
                 total_count=total_count,
                 parent=name,
                 decision=r_dec,
-                highlight=highlight and direction == "right"
+                highlight=highlight and direction == "right",
             )
         else:  # leaf
             shape = "ellipse"
             name = f"leaf{root['leaf_index']}"
             label = f"leaf {root['leaf_index']}: "
             label += f"<B>{_float2str(root['leaf_value'], precision)}</B>"
-            if 'leaf_weight' in show_info:
+            if "leaf_weight" in show_info:
                 label += f"<br/>{_float2str(root['leaf_weight'], precision)} weight"
-            if 'leaf_count' in show_info:
+            if "leaf_count" in show_info:
                 label += f"<br/>count: {root['leaf_count']}"
             if "data_percentage" in show_info:
                 label += f"<br/>{_float2str(root['leaf_count'] / total_count * 100, 2)}% of data"
             label = f"<{label}>"
-        graph.node(name, label=label, shape=shape, style=style, fillcolor=fillcolor, color=color, penwidth=penwidth, tooltip=tooltip)
+        graph.node(
+            name,
+            label=label,
+            shape=shape,
+            style=style,
+            fillcolor=fillcolor,
+            color=color,
+            penwidth=penwidth,
+            tooltip=tooltip,
+        )
         if parent is not None:
             graph.edge(parent, name, decision, color=color, penwidth=penwidth)
 
     graph = Digraph(**kwargs)
     rankdir = "LR" if orientation == "horizontal" else "TB"
     graph.attr("graph", nodesep="0.05", ranksep="0.3", rankdir=rankdir)
-    if "internal_count" in tree_info['tree_structure']:
+    if "internal_count" in tree_info["tree_structure"]:
         add(
-            root=tree_info['tree_structure'],
-            total_count=tree_info['tree_structure']["internal_count"],
+            root=tree_info["tree_structure"],
+            total_count=tree_info["tree_structure"]["internal_count"],
             parent=None,
             decision=None,
-            highlight=example_case is not None
+            highlight=example_case is not None,
         )
     else:
         raise Exception("Cannot plot trees with no split")
@@ -610,10 +616,10 @@ def create_tree_digraph(
     tree_index: int = 0,
     show_info: Optional[List[str]] = None,
     precision: Optional[int] = 3,
-    orientation: str = 'horizontal',
+    orientation: str = "horizontal",
     example_case: Optional[Union[np.ndarray, pd_DataFrame]] = None,
     max_category_values: int = 10,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Any:
     """Create a digraph representation of specified tree.
 
@@ -689,32 +695,32 @@ def create_tree_digraph(
     if isinstance(booster, LGBMModel):
         booster = booster.booster_
     elif not isinstance(booster, Booster):
-        raise TypeError('booster must be Booster or LGBMModel.')
+        raise TypeError("booster must be Booster or LGBMModel.")
 
     model = booster.dump_model()
-    tree_infos = model['tree_info']
-    feature_names = model.get('feature_names', None)
-    monotone_constraints = model.get('monotone_constraints', None)
+    tree_infos = model["tree_info"]
+    feature_names = model.get("feature_names", None)
+    monotone_constraints = model.get("monotone_constraints", None)
 
     if tree_index < len(tree_infos):
         tree_info = tree_infos[tree_index]
     else:
-        raise IndexError('tree_index is out of range.')
+        raise IndexError("tree_index is out of range.")
 
     if show_info is None:
         show_info = []
 
     if example_case is not None:
         if not isinstance(example_case, (np.ndarray, pd_DataFrame)) or example_case.ndim != 2:
-            raise ValueError('example_case must be a numpy 2-D array or a pandas DataFrame')
+            raise ValueError("example_case must be a numpy 2-D array or a pandas DataFrame")
         if example_case.shape[0] != 1:
-            raise ValueError('example_case must have a single row.')
+            raise ValueError("example_case must have a single row.")
         if isinstance(example_case, pd_DataFrame):
             example_case = _data_from_pandas(
                 data=example_case,
                 feature_name="auto",
                 categorical_feature="auto",
-                pandas_categorical=booster.pandas_categorical
+                pandas_categorical=booster.pandas_categorical,
             )[0]
         example_case = example_case[0]
 
@@ -727,21 +733,21 @@ def create_tree_digraph(
         constraints=monotone_constraints,
         example_case=example_case,
         max_category_values=max_category_values,
-        **kwargs
+        **kwargs,
     )
 
 
 def plot_tree(
     booster: Union[Booster, LGBMModel],
-    ax=None,
+    ax: "Optional[matplotlib.axes.Axes]" = None,
     tree_index: int = 0,
     figsize: Optional[Tuple[float, float]] = None,
     dpi: Optional[int] = None,
     show_info: Optional[List[str]] = None,
     precision: Optional[int] = 3,
-    orientation: str = 'horizontal',
+    orientation: str = "horizontal",
     example_case: Optional[Union[np.ndarray, pd_DataFrame]] = None,
-    **kwargs: Any
+    **kwargs: Any,
 ) -> Any:
     """Plot specified tree.
 
@@ -804,25 +810,31 @@ def plot_tree(
         The plot with single tree.
     """
     if MATPLOTLIB_INSTALLED:
-        import matplotlib.image as image
+        import matplotlib.image
         import matplotlib.pyplot as plt
     else:
-        raise ImportError('You must install matplotlib and restart your session to plot tree.')
+        raise ImportError("You must install matplotlib and restart your session to plot tree.")
 
     if ax is None:
         if figsize is not None:
-            _check_not_tuple_of_2_elements(figsize, 'figsize')
+            _check_not_tuple_of_2_elements(figsize, "figsize")
         _, ax = plt.subplots(1, 1, figsize=figsize, dpi=dpi)
 
-    graph = create_tree_digraph(booster=booster, tree_index=tree_index,
-                                show_info=show_info, precision=precision,
-                                orientation=orientation, example_case=example_case, **kwargs)
+    graph = create_tree_digraph(
+        booster=booster,
+        tree_index=tree_index,
+        show_info=show_info,
+        precision=precision,
+        orientation=orientation,
+        example_case=example_case,
+        **kwargs,
+    )
 
     s = BytesIO()
-    s.write(graph.pipe(format='png'))
+    s.write(graph.pipe(format="png"))
     s.seek(0)
-    img = image.imread(s)
+    img = matplotlib.image.imread(s)
 
     ax.imshow(img)
-    ax.axis('off')
+    ax.axis("off")
     return ax
